﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Insteon.Messages.Insteon
{
    internal class StandardMessage : Message
    {
        internal const int MessCommand = 0x50;
        internal const int MessLength = 11;
        internal const int MessFlagsByte = 8;
        internal const int MessSizeBit = 0;

        internal readonly InsteonAddress To;
        internal readonly InsteonAddress From;
        internal readonly byte Command1;
        internal readonly byte Command2;
        internal readonly Flags Flags;

        public StandardMessage(byte[] buffer, UInt16 Length, UInt64 Number, out UInt16 Size)
            : base(buffer, Length, Number)
        {
            if (!Check(buffer, Length))
            {
                throw new Exception("Invalid message format");
            }
            To = new InsteonAddress(buffer, 5);
            From = new InsteonAddress(buffer, 2);
            Command1 = buffer[9];
            Command2 = buffer[10];

            Flags = new Flags(buffer[MessFlagsByte]);
            Size = MessLength;
        }

        /// <summary>
        /// does the buffer contains a StandardMessage at the start of the buffer?
        /// </summary>
        /// <param name="buffer"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        internal static bool Check(byte[] buffer, UInt16 length)
        {
            if ((length >= MessLength) && (buffer[1] == MessCommand))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public override string ToString()
        {
            return PrintCommonMessage(this.ReceivedTime, Number, this, To, From, Flags, Command1, Command2, null);
        }

        internal override void RouteToDevice(InsteonInterface Interface)
        {
            Interface.RouteToTarget(this, To);
            Interface.RouteToTarget(this, From); 
        }

    }
}
